﻿
Module Sorting
    Interface IBucketSortable
        Function Transformation() As Integer
    End Interface

    Public Sub Bucketsort(Of T)(ByVal Values As T(), ByVal NumBuckets As Integer, ByVal Func As Func(Of T, Integer))
        Dim Length As Integer = Values.Length - 1

        Dim Buckets(NumBuckets) As List(Of T)

        ' Transformieren und die Eimer zuweisen
        For Each Item In Values
            Dim idx As Integer = Func(Item) ' Index
            If Buckets(idx) Is Nothing Then Buckets(idx) = New List(Of T)
            Buckets(idx).Add(Item)
        Next

        ' Aus den Eimern lesen
        Dim Index As Integer = 0
        For i = 0 To NumBuckets
            If Buckets(i) Is Nothing Then Continue For
            ' Solange etwas in das Array schreiben, bis der aktuelle Eimer leer ist
            While Buckets(i).Count > 0
                Values(Index) = Buckets(i)(Buckets(i).Count - 1)
                Index += 1
                Buckets(i).RemoveAt(Buckets(i).Count - 1) ' Löschen
            End While
        Next
    End Sub

    ' Wenn wir ein IBucketSortable haben, muss die Transformation nicht angegeben werden
    Public Sub Bucketsort(Of T As IBucketSortable)(ByVal Values As T(), ByVal NumBuckets As Integer)
        Bucketsort(Values, NumBuckets, Function(x As IBucketSortable) x.Transformation)
    End Sub
End Module

Module Module1

    Class Stadt
        Implements IBucketSortable

        Public Name, Bundesland As String

        Public Function Transformation() As Integer Implements Sorting.IBucketSortable.Transformation
            Static Bundesländer As String() = {"Baden-Württemberg", _
                                                "Bayern", _
                                                "Berlin", _
                                                "Brandenburg", _
                                                "Bremen", _
                                                "Hamburg", _
                                                "Hessen", _
                                                "Mecklenburg-Vorpommern", _
                                                "Niedersachsen", _
                                                "Nordrhein-Westfalen", _
                                                "Rheinland-Pfalz", _
                                                "Saarland", _
                                                "Sachsen", _
                                                "Sachsen-Anhalt", _
                                                "Schleswig-Holstein", _
                                                "Thüringen"}
        
            Return Array.IndexOf(Bundesländer, Bundesland)
        End Function

        Public Overrides Function ToString() As String
            Return String.Format("{0} ({1})", Name, Bundesland)
        End Function
    End Class

    Sub Main()
        Dim Werte As Integer() = {-14, 5, 14, 6, 3, 13, 3, 12, 0, 6, 2, 1}

        Console.WriteLine("Vor dem Sortieren")
        Array.ForEach(Werte, AddressOf Console.WriteLine)

        Sorting.Bucketsort(Werte, 30, Function(x) x + 14)

        Console.WriteLine("Nach dem Sortieren")
        Array.ForEach(Werte, AddressOf Console.WriteLine)

        Dim Städte As Stadt() = {New Stadt() With {.Name = "Dresden", .Bundesland = "Sachsen"}, _
                                    New Stadt() With {.Name = "Flensburg", .Bundesland = "Schleswig-Holstein"}, _
                                    New Stadt() With {.Name = "Pullach", .Bundesland = "Bayern"}, _
                                    New Stadt() With {.Name = "Dresden", .Bundesland = "Sachsen"}, _
                                    New Stadt() With {.Name = "Rostock", .Bundesland = "Mecklenburg-Vorpommern"}, _
                                    New Stadt() With {.Name = "München", .Bundesland = "Bayern"}, _
                                    New Stadt() With {.Name = "Leipzig", .Bundesland = "Sachsen"}, _
                                    New Stadt() With {.Name = "Passau", .Bundesland = "Bayern"}}

        Console.WriteLine()
        Console.WriteLine()

        Console.WriteLine("Städte nach Bundesländern sortiert")

        Sorting.Bucketsort(Städte, 16 - 1)
        Array.ForEach(Städte, AddressOf Console.WriteLine)

        Console.ReadKey()
    End Sub

End Module
